package cn.chnmuseum.party.web.controller;

import cn.chnmuseum.party.common.util.AddressUtil;
import cn.chnmuseum.party.model.Menu;
import cn.chnmuseum.party.model.Role;
import cn.chnmuseum.party.model.TVideoVisitor;
import cn.chnmuseum.party.service.TVideoVisitorService;
import cn.chnmuseum.party.vo.StatisticData;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import cn.chnmuseum.party.web.controller.base.BaseController;
import cn.chnmuseum.party.model.Statistic;
import cn.chnmuseum.party.service.StatisticService;
import cn.chnmuseum.party.common.enums.AuditStatusEnum;
import cn.chnmuseum.party.common.validator.groups.Add;
import cn.chnmuseum.party.common.validator.groups.Update;
import cn.chnmuseum.party.common.vo.GenericPageParam;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.stereotype.Controller;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <pre>
 * 大屏统计数据 前端控制器
 * </pre>
 *
 * @author Danny Lee
 * @since 2021-03-20
 */
@Slf4j
@RestController
@RequestMapping("/screen")
@Api(tags = {"统计数据"})
public class StatisticController extends BaseController {

    @Resource
    private StatisticService statisticService;

    @Autowired
    private TVideoVisitorService tVideoVisitorService;

    @GetMapping("/statistic")
    @ApiOperation(value = "统计数据", notes = "统计数据")
    public ResponseEntity<StatisticData> getScreenData() {
		return ResponseEntity.ok(new StatisticData());
    }


    @GetMapping("/recordVisitor")
    @ApiOperation(value = "记录视频访问者的城市", notes = "记录视频访问者的城市")
    public Map recordVisitor(String videoId, HttpServletRequest request){
        Map<String, String> resultMap = new HashMap<>();
        try {
            TVideoVisitor tVideoVisitor = new TVideoVisitor();
            if (StringUtils.isBlank(videoId)) {
                resultMap.put("resultCoed", "400");
                resultMap.put("message", "展板不能为空");
            }else {
                tVideoVisitor.setVideoId(StringUtils.trimToNull(videoId));
            }
            String ipAddress = AddressUtil.getIpAddress(request);
            if (StringUtils.isBlank(ipAddress)) {
                resultMap.put("resultCoed", "400");
                resultMap.put("message", "获取数据异常");
            }
            tVideoVisitor.setUserIp(ipAddress);
            Map<String, String> addressByIp = AddressUtil.getAddressByIp(ipAddress);
            if (StringUtils.isNotBlank(addressByIp.get("region"))) {
                tVideoVisitor.setAreaCode(addressByIp.get("regionCode"));
                tVideoVisitor.setArea(addressByIp.get("region"));
            }else if(StringUtils.isNotBlank(addressByIp.get("city"))){
                tVideoVisitor.setAreaCode(addressByIp.get("cityCode"));
                tVideoVisitor.setArea(addressByIp.get("city"));
            }else {
                tVideoVisitor.setAreaCode(addressByIp.get("proCode"));
                tVideoVisitor.setArea(addressByIp.get("pro"));
            }

            // 该条信息的创建时间和更新时间
            tVideoVisitor.setCreateTime(new Date());
            tVideoVisitor.setUpdateTime(new Date());

            boolean save = tVideoVisitorService.save(tVideoVisitor);
            if (!save) {
                // 新增失败, 500
                resultMap.put("resultCode", "500");
                resultMap.put("message", "服务器忙");
                return resultMap;
            }else{
                resultMap.put("resultCode", "200");
                resultMap.put("message", "成功");
                return resultMap;
            }
        } catch (Exception e) {
            resultMap.put("resultCode", "500");
            resultMap.put("message", "服务器忙");
            logger.error("新增成员错误!", e);
        }
        return resultMap;
    }

    @ApiOperation(value = "统计导览扫码次数", notes = "统计导览扫码次数")
    @ApiImplicitParam(name = "videoId", value = "导览视频id")
    @GetMapping("/getH5VideoStatistics")
    public Map getH5VideoStatistics(String videoId){
        LambdaQueryWrapper<TVideoVisitor> tVideoVisitorLambdaQueryWrapper = new LambdaQueryWrapper<>();
        tVideoVisitorLambdaQueryWrapper.eq(TVideoVisitor::getVideoId, videoId);
        int count = this.tVideoVisitorService.count(tVideoVisitorLambdaQueryWrapper);
        return getResult(count);
    }

}

